@@ -23,6 +23,8 @@ pub struct Interface {
     pub name: String,
     /// The address details of the interface.
     pub addr: IfAddr,
+    /// The index of the interface.
+    pub index: Option<u32>,
 }
 
 impl Interface {
@@ -31,6 +33,11 @@ impl Interface {
         self.addr.is_loopback()
     }
 
+    /// Check whether this is a link local interface.
+    pub fn is_link_local(&self) -> bool {
+        self.addr.is_link_local()
+    }
+
     /// Get the IP address of this interface.
     pub fn ip(&self) -> IpAddr {
         self.addr.ip()
@@ -55,6 +62,14 @@ impl IfAddr {
         }
     }
 
+    /// Check whether this is a link local interface.
+    pub fn is_link_local(&self) -> bool {
+        match *self {
+            IfAddr::V4(ref ifv4_addr) => ifv4_addr.is_link_local(),
+            IfAddr::V6(ref ifv6_addr) => ifv6_addr.is_link_local(),
+        }
+    }
+
     /// Get the IP address of this interface address.
     pub fn ip(&self) -> IpAddr {
         match *self {
@@ -80,6 +95,11 @@ impl Ifv4Addr {
     pub fn is_loopback(&self) -> bool {
         self.ip.octets()[0] == 127
     }
+
+    /// Check whether this is a link local address.
+    pub fn is_link_local(&self) -> bool {
+        self.ip.is_link_local()
+    }
 }
 
 /// Details about the ipv6 address of an interface on this host.
@@ -98,10 +118,19 @@ impl Ifv6Addr {
     pub fn is_loopback(&self) -> bool {
         self.ip.segments() == [0, 0, 0, 0, 0, 0, 0, 1]
     }
+
+    /// Check whether this is a link local address.
+    pub fn is_link_local(&self) -> bool {
+        let bytes = self.ip.octets();
+
+        bytes[0] == 0xfe && bytes[1] == 0x80
+    }
 }
 
 #[cfg(not(windows))]
 mod getifaddrs_posix {
+    use libc::if_nametoindex;
+
     use super::{IfAddr, Ifv4Addr, Ifv6Addr, Interface};
     use crate::posix::{self as ifaddrs, IfAddrs};
     use crate::sockaddr;
@@ -163,7 +192,19 @@ mod getifaddrs_posix {
             let name = unsafe { CStr::from_ptr(ifaddr.ifa_name) }
                 .to_string_lossy()
                 .into_owned();
-            ret.push(Interface { name, addr });
+            let index = {
+                let index = unsafe { if_nametoindex(ifaddr.ifa_name) };
+
+                // From `man if_nametoindex 3`:
+                // The if_nametoindex() function maps the interface name specified in ifname to its
+                // corresponding index. If the specified interface does not exist, it returns 0.
+                if index == 0 {
+                    None
+                } else {
+                    Some(index)
+                }
+            };
+            ret.push(Interface { name, addr, index });
         }
 
         Ok(ret)
@@ -192,10 +233,10 @@ mod getifaddrs_windows {
 
         for ifaddr in ifaddrs.iter() {
             for addr in ifaddr.unicast_addresses() {
-                if addr.dadstate != IpDadStatePreferred {
+                if addr.DadState != IpDadStatePreferred {
                     continue;
                 }
-                let addr = match sockaddr::to_ipaddr(addr.address.lp_socket_address) {
+                let addr = match sockaddr::to_ipaddr(addr.Address.lpSockaddr) {
                     None => continue,
                     Some(IpAddr::V4(ipv4_addr)) => {
                         let mut item_netmask = Ipv4Addr::new(0, 0, 0, 0);
@@ -203,19 +244,19 @@ mod getifaddrs_windows {
 
                         // Search prefixes for a prefix matching addr
                         'prefixloopv4: for prefix in ifaddr.prefixes() {
-                            let ipprefix = sockaddr::to_ipaddr(prefix.address.lp_socket_address);
+                            let ipprefix = sockaddr::to_ipaddr(prefix.Address.lpSockaddr);
                             match ipprefix {
                                 Some(IpAddr::V4(ref a)) => {
                                     let mut netmask: [u8; 4] = [0; 4];
                                     for (n, netmask_elt) in netmask
                                         .iter_mut()
                                         .enumerate()
-                                        .take((prefix.prefix_length as usize + 7) / 8)
+                                        .take((prefix.PrefixLength as usize + 7) / 8)
                                     {
                                         let x_byte = ipv4_addr.octets()[n];
                                         let y_byte = a.octets()[n];
                                         for m in 0..8 {
-                                            if (n * 8) + m > prefix.prefix_length as usize {
+                                            if (n * 8) + m > prefix.PrefixLength as usize {
                                                 break;
                                             }
                                             let bit = 1 << (7 - m);
@@ -254,7 +295,7 @@ mod getifaddrs_windows {
                         let mut item_netmask = Ipv6Addr::new(0, 0, 0, 0, 0, 0, 0, 0);
                         // Search prefixes for a prefix matching addr
                         'prefixloopv6: for prefix in ifaddr.prefixes() {
-                            let ipprefix = sockaddr::to_ipaddr(prefix.address.lp_socket_address);
+                            let ipprefix = sockaddr::to_ipaddr(prefix.Address.lpSockaddr);
                             match ipprefix {
                                 Some(IpAddr::V6(ref a)) => {
                                     // Iterate the bits in the prefix, if they all match this prefix
@@ -263,12 +304,12 @@ mod getifaddrs_windows {
                                     for (n, netmask_elt) in netmask
                                         .iter_mut()
                                         .enumerate()
-                                        .take((prefix.prefix_length as usize + 15) / 16)
+                                        .take((prefix.PrefixLength as usize + 15) / 16)
                                     {
                                         let x_word = ipv6_addr.segments()[n];
                                         let y_word = a.segments()[n];
                                         for m in 0..16 {
-                                            if (n * 16) + m > prefix.prefix_length as usize {
+                                            if (n * 16) + m > prefix.PrefixLength as usize {
                                                 break;
                                             }
                                             let bit = 1 << (15 - m);
@@ -296,9 +337,14 @@ mod getifaddrs_windows {
                     }
                 };
 
+                let index = match addr {
+                    IfAddr::V4(_) => ifaddr.ipv4_index(),
+                    IfAddr::V6(_) => ifaddr.ipv6_index(),
+                };
                 ret.push(Interface {
                     name: ifaddr.name(),
                     addr,
+                    index,
                 });
             }
         }
@@ -410,6 +456,13 @@ mod tests {
                 .filter(|interface| interface.is_loopback())
                 .count()
         );
+        // if index is set, it is non-zero
+        for interface in &ifaces {
+            if let Some(idx) = interface.index {
+                assert!(idx > 0);
+            }
+        }
+
         // one address of IpV4(127.0.0.1)
         let is_loopback =
             |interface: &&Interface| interface.addr.ip() == IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1));
@@ -425,6 +478,8 @@ mod tests {
                 if interface.addr.ip() == addr {
                     listed = true;
                 }
+
+                assert!(interface.index.is_some());
             }
             assert!(listed);
         }
